-- name=int_diary
--******************************************************************************************
function public.Init ( pos_z )

  ObjAttach( "int_diary", interface.originhub );
  ObjSet( "int_diary", { pos_z = pos_z } );

  public.show = 0;
  private.SHOW_ANIM_TIME = 1;
  private.MOUSE_OVER_ANIM_TIME = 0.3;

  -------------------diary-----------------------------------------------------------
  private.page_count = 11;

  private.CountOfMarks = 11;
  -----------------------------------------------------------------------------------

  local table_multiset = {};

  for i = 1,private.CountOfMarks do 

    table.insert( table_multiset, 
                  {
                    "spr_diary_book_mark_"..i, 
                    {
                      event_mdown = function () private.OpenBookmark(i); end,
                      event_menter = function () private.MouseEnterAnim("button_"..i) end,
                      event_mleave = function () private.MouseLeaveAnim("button_"..i); end
                    }
                  } 
    );

  end;

  table.insert( table_multiset, 
                {
                  "spr_diary_close", 
                  { 
                    event_mdown = function () private.HideAnim('hide'); end,
                    event_menter = function () private.MouseEnterAnim("close_focus") end,
                    event_mleave = function () private.MouseLeaveAnim("close_focus"); end
                  }
                }
  );
  table.insert( table_multiset, 
                {
                  "spr_diary_events", 
                  {
                    event_mdown = function () private.OpenEvents(); end,
                    event_menter = function () SetCursor(CURSOR_HAND); end,
                    event_mleave = function () SetCursor(CURSOR_DEFAULT); end
                  }
                }
  );
  table.insert( table_multiset, 
                {
                  "spr_diary_participants", 
                  {
                    event_mdown = function () private.OpenBookmark(1); end,
                    event_menter = function () SetCursor(CURSOR_HAND); end,
                    event_mleave = function () SetCursor(CURSOR_DEFAULT); end
                  }
                }
  );
  table.insert( table_multiset, 
                {
                  "obj_diary_add_arrow_right", 
                  {
                    event_mdown = function () private.DiaryPageDown( "right" ) end,
                    event_menter = function () private.MouseEnterAnim("arrow_right") end,
                    event_mleave = function () private.MouseLeaveAnim("arrow_right"); end
                  }
                }
  );
  table.insert( table_multiset, 
                {
                  "obj_diary_add_arrow_left", 
                  {
                    event_mdown = function ()private.DiaryPageDown( "left" ) end,
                    event_menter = function () private.MouseEnterAnim("arrow_left") end,
                    event_mleave = function () private.MouseLeaveAnim("arrow_left"); end
                  }
                }
  );
  
  ObjMultiSet( table_multiset );

end;
--******************************************************************************************
function public.Destroy()

end;
--******************************************************************************************
function private.AnimEnd( event_id )
 
  common_impl.Lock(false);

  if ( event_id == "show" ) then
  
    public.show = 1;
    ObjSet( "obj_int_diary", { input = true } );

  elseif ( event_id == "hide" ) then
  
    public.show = 0;
    ObjSet( "obj_int_diary", { active = false, visible = false, input = false } );

  end;

end;
--******************************************************************************************
function private.ShowAnim( event_id )

  local trg = function () private.AnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  
  private.InitState();
  local prg = ng_global.currentprogress;

  if tostring(prg) == tostring('ext') then

    ObjMultiSet({
      { "obj_diary_book_mark" ,        { input = false, active = false, alp = 0 }},
      { "obj_diary_book_mark_right" ,  { input = false, active = false, alp = 0 }},
      { "obj_diary_participants_hub" , { input = false, active = false, alp = 0 }},
      { "spr_diary_back" ,             { res ="assets/interface/resources/diary/layers/back4lvl" }},      
    });

  end;

  common_impl.Lock(true);

  ObjAnimate( "obj_int_diary", "alp", 0, 0, trg,
  {
    0.0, 3, 0,
    tme, 3, 1
  } );

end;
--******************************************************************************************
function private.HideAnim( event_id )

  local trg = function () private.AnimEnd( event_id ); end;
  local tme = private.SHOW_ANIM_TIME;
  common_impl.Lock(true);
  ObjAnimate( "obj_int_diary", "alp", 0, 0, trg,
  {
    0.0, 3, 1,
    tme, 3, 0
  } );

end;
--******************************************************************************************
function private.MouseEnterAnim(but_name)

  SetCursor( CURSOR_HAND );

  local obj = "spr_diary_add_"..but_name;
  local alp = ObjGet( obj ).alp;
  ObjAnimate( obj, "alp", 0, 0, "", { 0, 0, alp, ( common_impl.animtime * ( 1 - alp ) ), 0, 1 } );

end;
--******************************************************************************************
function private.MouseLeaveAnim(but_name)

  SetCursor( CURSOR_DEFAULT );

  local obj = "spr_diary_add_"..but_name;
  local alp = ObjGet( obj ).alp;
  ObjAnimate( obj, "alp", 0, 0, "", { 0, 0, alp, ( common_impl.animtime * alp ), 0, 0 } );

end;
--******************************************************************************************
function private.InitState()

  private.AddBookmarks();
  private.AddEvents();

  local prg = ng_global.currentprogress;
  local last_location = ng_global.progress[ prg ].common.diary.last_location;

  if last_location == 0 then     -- в последний раз открывали вкладку события
    private.OpenEvents();
  else             -- открывали закладки, находим последнюю открытую закладку
    local last_bookmark = ng_global.progress[ prg ].common.diary.last_bookmark;
    private.OpenBookmark(last_bookmark);
  end;

end;
--******************************************************************************************
function private.DiaryPageDown( course )

  local func_after = function ()

    local prg = ng_global.currentprogress;
    ObjSet( "obj_diary_group_"..( ng_global.progress[ prg ].common.diary.current ), { alp = 0, visible = 0 } );

    if course == "left" then
      ng_global.progress[ prg ].common.diary.current = ng_global.progress[ prg ].common.diary.current - 1;
    else
      ng_global.progress[ prg ].common.diary.current = ng_global.progress[ prg ].common.diary.current + 1;
    end;

    ng_global.progress[ prg ].common.diary.last_location = 0; --  events = 0, participants = 1
    private.ButtonActive();
    common_impl.Lock(false);

  end;

  common_impl.Lock(true);
  local def = 0;
  if course == "left" then
    def = -1;
  else
    def = 1;
  end;

  local prg = ng_global.currentprogress;

  ObjAnimate( "obj_diary_group_"..ng_global.progress[ prg ].common.diary.current, "alp", 0, 0, "", 
  { 
    0.0, 0, 1, 
    0.2, 0, 0, 
  } );

  ObjSet( "obj_diary_group_"..( ng_global.progress[ prg ].common.diary.current + def ), { alp = 0, visible = 1 } );

  ObjAnimate( "obj_diary_group_"..( ng_global.progress[ prg ].common.diary.current + def ), "alp", 0, 0, func_after, 
  { 
    0.0, 0, 0, 
    0.2, 0, 1 
  } );

  local new_song;
  new_song = math.random(1,3);

  common_impl.PlayAudio( "sfx", "common/audio/aud_turn_page_"..new_song, 0 );

end;

--******************************************************************************************
function private.ButtonActive()

  local prg = ng_global.currentprogress;
  local right_index = math.ceil( ng_global.progress[ prg ].common.diary.last_page / 2 );
  local left_index = 1; 

  if ng_global.progress[ prg ].common.diary.current ~= "" then
    
    local num = tonumber( ng_global.progress[ prg ].common.diary.current );

    if num < tonumber( right_index ) then
      ObjSet( "obj_diary_add_arrow_right", { input = 1, alp = 1 } );
    else
      ObjSet( "obj_diary_add_arrow_right", { input = 0, alp = 1 } );
    end;
    
    if num > tonumber( left_index ) then
      ObjSet( "obj_diary_add_arrow_left", { input = 1, alp = 1 } );
    else
      ObjSet( "obj_diary_add_arrow_left", { input = 0, alp = 1 } );
    end;    
  end; 

end;
--******************************************************************************************
function private.OpenDiaryPart( part )

    private.OpenBookmark(1);

end;
--******************************************************************************************
  private.events_small_state = -305;
  private.participants_small_state = -305;

  local offset = 20;
  local offset_events = 12;
  local offset_participants = 18;

--******************************************************************************************
function private.AddBookmarks()

  ObjSet("spr_diary_events", { input = 1 });
  ObjSet("spr_diary_participants", { input = 1 });

  local prg = ng_global.currentprogress;
  local count = #ng_global.progress[ prg ].common.diary.bookmarks;
  
  for i = 1, #ng_global.progress[ prg ].common.diary.bookmarks  do
    ObjSet("spr_diary_book_mark_"..i, { alp = 1, input = 1});      
  end;

  local page_pos = { 291, 688 };

  for i = 1, count do
    ObjSet( "obj_participants_group_"..i, { alp = 0, visible = 0 } );
  end;

  local last_location = ng_global.progress[ prg ].common.diary.last_location;
  local last_bookmark = ng_global.progress[ prg ].common.diary.last_bookmark;

  if last_location == 1 then
    ObjSet( "obj_participants_group_"..last_bookmark, { alp = 1, visible = 1 } );
  else
    ObjSet( "obj_participants_group_"..last_bookmark, { alp = 0, visible = 1} );
  end;
  -- i - bookmark number
  -- j - page number in bookmark

  for i = 1, count  do
    
    local bookmark = ng_global.progress[ prg ].common.diary.bookmarks[i]; 

    for j = 1, #bookmark do
           
      ObjAttach("obj_bookmark"..i.."_page"..j, "obj_participants_group_"..i);
      ObjSet( "obj_bookmark"..i.."_page"..j, { pos_x = page_pos[j], alp = 1, visible = 1 } );

      if (bookmark[j][1] ~= 0) then  --header
        ObjSet("txt_diary_bookmark"..i.."_header_"..j, {alp = 1});
        ObjSet("spr_diary_bookmark"..i.."_strike_"..j, {alp = 1});
           
      end;

      if bookmark[j][2] ~= 0 then   --img
        ObjSet("spr_diary_bookmark"..i.."_page_"..j, {alp = 1});
         
      end;

      if bookmark[j][3] ~= 0 then  --txt
        ObjSet("txt_diary_bookmark"..i.."_page_"..j, {alp = 1});
        
      end;
    end;
  end; 

end;
--******************************************************************************************
function private.OpenBookmark(bookmark_number)

  local func_after = function ()

    ObjSet("obj_diary_book_mark_right", {alp = 1, input = 1});

    ObjSet("obj_diary_arrows", {alp = 0, input = 0});
    ObjSet("obj_diary_arrows_obj", {alp = 0, input = 0 });
    
    local prg = ng_global.currentprogress;
    ng_global.progress[ prg ].common.diary.last_location = 1; --  events = 0, participants = 1
    common_impl.Lock(false);

  end;

  local prg = ng_global.currentprogress;
  local bookmark = ng_global.progress[ prg ].common.diary.bookmarks[bookmark_number];
  local last_bookmark = ng_global.progress[ prg ].common.diary.last_bookmark;

  ObjSet("spr_diary_participants", { pos_y = private.participants_small_state - offset_participants});   
  common_impl.Lock(true);

  -- переход со страницы дневника событий на страницу закладок
  if ng_global.progress[ prg ].common.diary.last_location == 0 then
    ObjSet("spr_diary_events", { pos_y = private.events_small_state });
    
    ObjAnimate( "obj_diary_group_"..ng_global.progress[ prg ].common.diary.current, "alp", 0, 0, "", 
    { 
      0.0, 0, 1, 
      0.2, 0, 0 
    } );

    ObjSet( "obj_participants_group_"..bookmark_number, { alp = 0, visible = 1 } );


    ObjAnimate( "obj_participants_group_"..bookmark_number, "alp", 0, 0, func_after, 
    { 
      0.0, 0, 0, 
      0.2, 0, 1 
    } );

    ObjAnimate( "obj_diary_arrows", "alp", 0, 0, "", 
    { 
      0.0, 0, 1, 
      0.2, 0, 0 
    } );

    ObjAnimate( "obj_diary_book_mark_right", "alp", 0, 0, "", 
    { 
      0.0, 0, 0, 
      0.2, 0, 1 
    } );

  -- переход с одной страницы закладок на другую
  elseif ng_global.progress[ prg ].common.diary.last_location == 1  and bookmark_number ~= last_bookmark then
    ObjAnimate( "obj_participants_group_"..ng_global.progress[ prg ].common.diary.last_bookmark, "alp", 0, 0, "", 
    { 
      0.0, 0, 1, 
      0.2, 0, 0 
    } );

    ObjSet( "obj_participants_group_"..bookmark_number, { alp = 0, visible = 1 } );
    ObjAnimate( "obj_participants_group_"..bookmark_number, "alp", 0, 0, func_after, 
    { 
      0.0, 0, 0, 
      0.2, 0, 1 
    } );
  else 

    ObjSet("obj_diary_arrows",     {alp = 0, input = 0 });
    ObjSet("obj_diary_arrows_obj", {alp = 0, input = 0 });

    ObjSet("obj_diary_book_mark_right", {alp = 1, input = 1});

    local prg = ng_global.currentprogress;
    ng_global.progress[ prg ].common.diary.last_location = 1; --  events = 0, participants = 1
    common_impl.Lock(false);

  end;
  
  ng_global.progress[ prg ].common.diary.last_bookmark = bookmark_number;

  local new_song;
  new_song = math.random(1,3);

  common_impl.PlayAudio( "sfx", "common/audio/aud_turn_page_"..new_song, 0 );

end;
--******************************************************************************************
function private.OpenEvents()

  local func_after = function ()

    ObjSet("obj_diary_book_mark_right", {alp = 0, input = 0});

    ObjSet("obj_diary_arrows", {alp = 1});
    ObjSet("txt_diary_slash",  {alp = 1});

    ObjSet("obj_diary_arrows_obj", {alp = 1, input=1});

    private.ButtonActive();
    common_impl.Lock(false);

  end;

  local prg = ng_global.currentprogress;

  common_impl.Lock(true); 
  local last_bookmark = ng_global.progress[ prg ].common.diary.last_bookmark;

  ObjSet("spr_diary_events", { pos_y = private.events_small_state - offset_events });

   local new_song;
   new_song = math.random(1,3);

  if ng_global.progress[ prg ].common.diary.last_location == 1 then 
    ObjSet("spr_diary_participants", { pos_y = private.participants_small_state}); 

    ObjAnimate( "obj_participants_group_"..last_bookmark, "alp", 0, 0, "", 
    { 
      0.0, 0, 1, 
      0.2, 0, 0 
    } );
    ObjSet( "obj_diary_group_"..ng_global.progress[ prg ].common.diary.current, { alp = 0, visible = 1 } );
    
    ObjAnimate( "obj_diary_group_"..ng_global.progress[ prg ].common.diary.current, "alp", 0, 0, func_after, 
    { 
      0.0, 0, 0, 
      0.2, 0, 1 
    } );

    ObjAnimate( "obj_diary_arrows", "alp", 0, 0, "", 
    { 
      0.0, 0, 0, 
      0.2, 0, 1 
    } );  

    ObjAnimate("obj_diary_book_mark_right", "alp", 0, 0, "", 
    { 
      0.0, 0, 1, 
      0.2, 0, 0 
    } );  
    
  else  
    ObjSet("obj_diary_arrows", {alp = 1});
    ObjSet("txt_diary_slash", {alp = 1});

    ObjSet("obj_diary_arrows_obj", {alp = 1, input=1});
    
    private.ButtonActive();
    common_impl.Lock(false);
    ObjSet("obj_diary_book_mark_right", {alp = 0, input = 0});
    
  end;
  ng_global.progress[ prg ].common.diary.last_location = 0;

  local new_song;
  new_song = math.random(1,3);

  common_impl.PlayAudio( "sfx", "common/audio/aud_turn_page_"..new_song, 0 );
  

end;
--******************************************************************************************
function private.AddEvents()

  local prg = ng_global.currentprogress;
  local current = ng_global.progress[ prg ].common.diary.current;
  local page_pos = { 291, 688 };

  for i = 1, math.ceil( private.page_count / 2 ) do
    ObjSet( "obj_diary_group_"..i, { alp = 0, visible = 0 } );
  end;
  
  local last_location = ng_global.progress[ prg ].common.diary.last_location
  if last_location == 0 then
    ObjSet( "obj_diary_group_"..current, { alp = 1, visible = 1 } );
  else
    ObjSet( "obj_diary_group_"..current, { alp = 0, visible = 1} );
  end;

  for i = 1, ng_global.progress[ prg ].common.diary.last_page  do
    local group = math.ceil( i / 2 );
    local page = math.fmod( i , 2 );
    if page == 0 then
      page = 2;
    end;
    ObjAttach("obj_diary_page_"..ng_global.progress[ prg ].common.diary.pages[ i ],"obj_diary_group_"..group);
    ObjSet( "obj_diary_page_"..ng_global.progress[ prg ].common.diary.pages[ i ], { pos_x = page_pos[ page ], alp = 1, visible = 1 } );
  end;

end;
--******************************************************************************************
function public.Show()

  local event_id = "show";
  
  ObjSet( "obj_int_diary", { active = true, visible = true, alp = 0 } );
  common_impl.HideDeployInv();
  private.ShowAnim( event_id );

end;
--******************************************************************************************
function public.Hide()

  local event_id = "hide";
  private.HideAnim( event_id );

end;
--******************************************************************************************