-- name=int_panel_providence
--******************************************************************************************
function public.Init ( pos_z )

  ObjAttach( "int_panel_providence", "obj_int_inventory_providence" );
  ObjSet( "int_panel_providence", { pos_x = 0, pos_y = 0, pos_z = pos_z } );

  private.ITEM_POSITION_OFFSET = 66;

  private.ITEM_FLY_ANIM_TIME  = 0.5;
  private.ITEM_MOVE_ANIM_TIME = 0.3;
  private.ITEM_DRAG_ANIM_TIME = 0.1;
  private.ITEM_DROP_ANIM_TIME = 0.2;

  private.ITEM_MOUSE_OVER_ANIM_TIME = 0.15;

  private.ITEM_SCALE       = 0.6;
  private.ITEM_SCALE_FOCUS = 0.7;
  private.ITEM_SCALE_FLY   = 1.0;

  private.DONT_DELETE_ITEMS = true;

  private.MOUSE_OVER_ANIM_TIME = 0.3;
  private.MOVE_ANIM_TIME      =  0.6;

  private.is_objects_moving = false;
  private.is_opened_for_taskheader = false;
  private.count_visible_objects = 2;
  private.m_left_object_index = 0; -- left visible object index
  private.m_objects = {};
  private.m_objects_params = {};
  private.m_dragging_object = nil;

  private.item_positions = {};

  private.event_anim_end = {};

  ObjSet("spr_int_panel_providence", {
    event_menter = function () int_inventory.Show( true ); end,
    event_mleave = function () int_inventory.Hide( false ); end
  });

end;
--******************************************************************************************
function public.Destroy()

end;
--******************************************************************************************
function private.EventAnimEnd( event_id )

  local event_params = private.event_anim_end[ event_id ];

  if ( event_id == "show" ) then

    ObjSet( "int_panel_providence", { input = true } );

  elseif ( event_id == "hide" ) then

    ObjSet( "int_panel_providence", { active = false, visible = false, input = false } );

  elseif ( string.find( event_id, "item_fly" ) ) then

  local item_name      = event_params[ "name" ];
  local item_game_name = event_params[ "game_name" ];
  local item_count     = event_params[ "count" ];

  ObjDelete( item_game_name );
  private.ExecuteAddEndFunc( item_name );

  int_inventory.count_fly_animation_playing = int_inventory.count_fly_animation_playing - 1;
  local fly_def_name = interface.GetObjectName( item_name ).."_"..item_count;
  local fly_obj_name = "fly_int_providence_panel_"..fly_def_name;

  ObjDelete( fly_obj_name );

  int_inventory.Hide();

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "item_move" ) ) then

    private.is_objects_moving = false;

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "item_drag" ) ) then

  private.m_dragging_object = item_name;
  ObjSet( private.m_dragging_object, { input = true } );

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "item_drop" ) ) then

  local item_name = event_params[ "name" ];

  ObjSet( item_name, { input = true } );
  int_inventory.Hide();

------------------------------------------------------------------------------------
  end;

end;
--******************************************************************************************
function private.ShowAnim( event_id )

  local obj = "int_panel_providence";
  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.MOVE_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 2, 0,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.HideAnim( event_id )

  local obj = "int_panel_providence";
  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = private.MOVE_ANIM_TIME;

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 2, 1,
    tme, 2, 0
  } );

end;
--******************************************************************************************
function public.Show( event_id )
  
  ObjSet( "int_panel_providence", { active = true, visible = true, input = true } );
  ObjSet( "spr_int_inventory_side_right", { visible = false } );
  ObjSet( "spr_int_inventory", { pos_x = -87 } );
  private.ShowAnim( "show" );
  
end;
--******************************************************************************************
function public.Hide( event_id )

  private.HideAnim( "hide" );

end;
--******************************************************************************************
function private.StdItemFlyAnimation( anim_obj_name, target_obj_name, trigger_end, first_item )

  first_item = first_item or 0;

  MsgSend( Command_Transporter_MoveObject,
  {
  type       = Trans_SceneToInventory,
  obj_name   = anim_obj_name,
  toobj_name = target_obj_name,
  show       = first_item,
  end_trig   = trigger_end
  } );

end;
--******************************************************************************************
function private.StdItemDragAnimation( anim_obj_name, target_obj_name, trigger_end )

  MsgSend( Command_Transporter_MoveObject,
  {
  type       = Trans_InventoryToCursor,
  obj_name   = anim_obj_name,
  toobj_name = target_obj_name,
  end_trig   = trigger_end
  } );

end;
--******************************************************************************************
function private.StdItemDropAnimation( anim_obj_name, target_obj_name, trigger_end, scale_end )

  if ( ObjGet( anim_obj_name ) ) then

    MsgSend( Command_Transporter_MoveObject,
    {
    type       = Trans_CursorToInventory,
    obj_name   = anim_obj_name,
    toobj_name = target_obj_name,
    scale      = scale_end,
    end_trig   = trigger_end
    } );

  else

    trigger_end();

  end;

end;
--******************************************************************************************
function public.AddObject( item_name, item_game_name, silhouette, func )
  
  ObjSet( item_game_name, { input = false } );

  local answer = MsgSend( Request_Level_IsLoading, {} );
  if answer.loading == 1 then

    ObjDelete( item_game_name );
    item_game_name = "";    

  end;

  local index = 0;
  for i = 1, #private.m_objects do
    if private.m_objects[ i ].name == item_name then
      private.m_objects[ i ].count = private.m_objects[ i ].count + 1;
      index = i;
      break;
    end;
  end;

  if index == 0 then
    table.insert( private.m_objects, { name = item_name, count = 1 } );
    index = #private.m_objects;

    private.m_objects_params[ item_name ] = {};
    local item_params = ObjGet( item_name );

    if item_params.event_startdrag == nil then
      item_params.event_startdrag = function () end;
    end;
    private.m_objects_params[ item_name ].event_startdrag = item_params.event_startdrag;

    if item_params.event_dragdrop == nil then
      item_params.event_dragdrop = function () end;
    end;
    private.m_objects_params[ item_name ].event_dragdrop = item_params.event_dragdrop;

    if item_params.event_menter == nil then
      item_params.event_menter = function () end;
    end;
    private.m_objects_params[ item_name ].event_menter = item_params.event_menter;

    if item_params.event_mleave == nil then
      item_params.event_mleave = function () end;
    end;
    private.m_objects_params[ item_name ].event_mleave = item_params.event_mleave;

    ObjSet( item_name, { 
      drag = true,
      event_startdrag = function () private.StartDrag( item_name ); end,
      event_dragdrop  = function () private.StopDrag( item_name ); end,
      event_menter    = function () private.ItemMouseEnter( item_name ); end,
      event_mleave    = function () private.ItemMouseLeave( item_name ); end
    } );
  end;

  private.m_objects[ index ].func = func;
  if index - 1 <= private.m_left_object_index then
    private.m_left_object_index = index - 1;

  elseif index - 1 > private.m_left_object_index then
    while index - 1 >= private.m_left_object_index + private.count_visible_objects do
      private.m_left_object_index = private.m_left_object_index + 1;
    end;

  end;

  local item_count = private.m_objects[ index ].count;
  local item_pos = private.InvGetObjPos( index - 1, private.m_left_object_index, private.count_visible_objects );

  --DbgTrace( "INV ADD: "..item_name.." ( "..item_game_name.." ), pos = "..item_pos..", count = "..item_count );

  local def_name = interface.GetObjectName( item_name );
  local hub_name = "hub_int_providence_panel_"..def_name;
  local sub_name = "sub_int_providence_panel_"..def_name;
  local obj_name = "itm_int_providence_panel_"..def_name;

  --if ( item_count == 1 ) then
    ObjDelete( hub_name );

    -- основной хаб ( движение по позиции, наведение )
    ObjCreate( hub_name, "obj" );
    ObjSet( hub_name,
    {
      pos_x = ( private.ITEM_POSITION_OFFSET * item_pos ),
      pos_y = 0
    } );

    if ( silhouette ) then

      ObjSet( hub_name, { input = false });

    end;

    private.item_positions[ item_name ] = item_pos;

    ObjAttach( hub_name, "obj_int_panel_providence" );

    -- под-хаб ( появление после полета, исчезание при драге )
    ObjCreate( sub_name, "obj" );
    ObjAttach( sub_name, hub_name );

    -- объект предмета
    ObjCreate( obj_name, "obj" );
    ObjAttach( obj_name, sub_name );

    -- спрайт предмета
    local new_item_name = item_name;
    if ( silhouette ) then

      new_item_name = new_item_name.."_silhouette";

    end;

    ObjSet( new_item_name, { input = true, pos_x = 0, pos_y = 0 } );
    ObjAttach( new_item_name, obj_name );

    local item_params = ObjGet( item_name );

    ObjSet( obj_name,
    {
      pos_z = 13,

      scale_x = private.ITEM_SCALE,
      scale_y = private.ITEM_SCALE,

      --inputrect_init = 1,
      --inputrect_x    = - ( 0.5 * item_params.draw_width  ),
      --inputrect_y    = - ( 0.5 * item_params.draw_height ),
      --inputrect_w    = item_params.draw_width,
      --inputrect_h    = item_params.draw_height
    } );

    -- бэк
    private.ItemCreate( new_item_name, sub_name );

  --end;

  if ( item_game_name == "" ) then

    private.ExecuteAddEndFunc( item_name );

  else

    local fly_def_name = interface.GetObjectName( item_name ).."_"..item_count;
    local fly_obj_name = "fly_int_providence_panel_"..fly_def_name;

    ObjCreate( fly_obj_name, "obj" );
    ObjAttach( fly_obj_name, InterfaceWidget_Top_Name );

    local item_game_pos = GetObjPosByObj( item_game_name );

    ObjSet( fly_obj_name,
    {
      input      = 0,
      pos_x      = item_game_pos[ 1 ],
      pos_y      = item_game_pos[ 2 ]
    } );

    if silhouette then

      local anm_obj_name = "anm_int_providence_panel_"..fly_def_name;
      ObjCreate( anm_obj_name, "partsys" );

      ObjSet( anm_obj_name, { 
        pos_x = 0, pos_y = 0, pos_z = -1,
        res = "assets/interface/resources/providence_panel/anims/anm_providence_fly"
      } );

      ObjAttach( anm_obj_name, fly_obj_name );

    end;

    ObjSet( item_game_name, { pos_x = 0, pos_y = 0 } );
    ObjAttach( item_game_name, fly_obj_name );

    int_inventory.count_fly_animation_playing = int_inventory.count_fly_animation_playing + 1;
    
    local event_id = "item_fly_"..item_name.."_"..item_count;
    private.event_anim_end[ event_id ] = { name = item_name, 
                                           game_name = item_game_name, 
                                           pos = item_pos, 
                                           count = item_count, 
                                           func = func };

    private.ItemFlyAnim( fly_obj_name, sub_name, item_count, event_id );

  end;

  int_inventory.Show();
  private.MoveObjects();

end;
--******************************************************************************************
function public.DeleteObject( item_name )

  local index = 0;
  for i = 1, #private.m_objects do
    if private.m_objects[ i ].name == item_name then
      index = i;
      break;
    end;
  end;

  if index ~= 0 then

    if item_name == private.m_dragging_object then

      if item_name == private.m_dragging_object then

        local def_name = interface.GetObjectName( item_name );
        local obj_name = "itm_int_inventory_"..def_name;

        MsgSend( Command_Transporter_StopAnim, 
        { 
          type = Trans_InventoryToCursor,
          obj_name = obj_name
        } );
      end;
      private.m_dragging_object = nil;
      SetCursor( CURSOR_DEFAULT );
    end;
    ObjDetach( item_name );
    --A.K

    if index - 1 < private.m_left_object_index then
      private.m_left_object_index = private.m_left_object_index - 1;

    elseif index - 1 >= private.m_left_object_index and 
           index - 1 < private.m_left_object_index + private.count_visible_objects then
      if private.m_left_object_index > 0 then
        private.m_left_object_index = private.m_left_object_index - 1;
      end;

    end;

    table.remove( private.m_objects, index );

    ObjSet( item_name, private.m_objects_params[ item_name ] );

    private.m_objects_params[ item_name ] = nil;
    private.MoveObjects();

    local def_name = interface.GetObjectName( item_name );
    local hub_name = "hub_int_providence_panel_"..def_name;
    local sub_name = "sub_int_providence_panel_"..def_name;
    local obj_name = "itm_int_providence_panel_"..def_name;

    if ( private.DONT_DELETE_ITEMS ) then

      ObjAttach( item_name, "int_panel_providence_items_hub" );

    else

      ObjDelete( item_name );

    end;

    ObjDelete( obj_name );
    ObjDelete( hub_name );

  end;

end;
--******************************************************************************************
function private.ShowObject( item_name )

  local index = 0;
  for i = 1, #private.m_objects do
    if private.m_objects[ i ].name == item_name then
      index = i;
      break;
    end;
  end;

  if index ~= 0 then

    if index - 1 < private.m_left_object_index then
      private.m_left_object_index = index - 1;
      private.MoveObjects();

    elseif index - 1 >= private.m_left_object_index + private.count_visible_objects then
      private.m_left_object_index = index - private.count_visible_objects;
      private.MoveObjects();

    end;

  end;

end;
--******************************************************************************************
function private.ItemFlyAnim( fly_name, item_name, item_count, event_id )

  local first_item = 0;

  --if ( item_count == 1 ) then

    first_item = 1;

  --end;

  local trg = function () private.EventAnimEnd( event_id ); end;

  private.StdItemFlyAnimation( fly_name, item_name, trg, first_item );

end;
--******************************************************************************************
function private.ZoomIn( item_name )

  local def_name = interface.GetObjectName( item_name );
  local hub_name = "hub_int_providence_panel_"..def_name;
  local sub_name = "sub_int_providence_panel_"..def_name;
  local obj_name = "itm_int_providence_panel_"..def_name;

  local event_id = "item_menter";
  private.event_anim_end[ event_id ] = {};

  private.ItemMouseEnterAnim( obj_name, event_id );

end;
--******************************************************************************************
function private.ZoomOut( item_name )

  local def_name = interface.GetObjectName( item_name );
  local hub_name = "hub_int_providence_panel_"..def_name;
  local sub_name = "sub_int_providence_panel_"..def_name;
  local obj_name = "itm_int_providence_panel_"..def_name;

  local event_id = "item_mleave";
  private.event_anim_end[ event_id ] = {};

  private.ItemMouseLeaveAnim( obj_name, event_id );

end;
--******************************************************************************************
function private.ItemMouseEnterAnim( item_name, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;

  local cur_scale_x = ObjGet( item_name ).scale_x;
  local cur_scale_y = ObjGet( item_name ).scale_y;

  local end_scale_x = private.ITEM_SCALE_FOCUS;
  local end_scale_y = private.ITEM_SCALE_FOCUS;

  local tme = private.ITEM_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( item_name, "scale_xy", 0, 0, trg,
  {
    0.0, 3, cur_scale_x, cur_scale_y,
    tme, 3, end_scale_x, end_scale_x
  } );

end;
--******************************************************************************************
function private.ItemMouseLeaveAnim( item_name, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;

  local cur_scale_x = ObjGet( item_name ).scale_x;
  local cur_scale_y = ObjGet( item_name ).scale_y;

  local end_scale_x = private.ITEM_SCALE;
  local end_scale_y = private.ITEM_SCALE;

  local tme = private.ITEM_MOUSE_OVER_ANIM_TIME;

  ObjAnimate( item_name, "scale_xy", 0, 0, trg,
  {
    0.0, 3, cur_scale_x, cur_scale_y,
    tme, 3, end_scale_x, end_scale_y
  } );

end;
--******************************************************************************************
function private.MoveObjects()

  private.is_objects_moving = true;
  local size = private.count_visible_objects;
  local objects = {};

  for i = 1, #private.m_objects do
    table.insert( objects, { name = private.m_objects[ i ].name, 
                             pos = private.InvGetObjPos( i - 1, 
                                                         private.m_left_object_index, 
                                                         private.count_visible_objects ) } );
  end;

  --DbgTrace( "INV MOVE: size = "..size..", objects count = "..#objects );

  for i = 1, #objects, 1 do

    local item_name = objects[ i ][ "name" ];
    local item_pos  = objects[ i ][ "pos" ];

    local hub_name = "hub_int_providence_panel_"..interface.GetObjectName( item_name );

    local move_type = 0;

    local item_pos_old = private.item_positions[ item_name ];--( ObjGet( hub_name ).pos_x ) / interface.inventory.ITEM_POSITION_OFFSET;

    private.item_positions[ item_name ] = item_pos;

    local move_pos = private.ITEM_POSITION_OFFSET * item_pos;

    if     ( ( item_pos_old >  - 1 ) and ( item_pos ==  - 1 ) )
    or     ( ( item_pos_old < size ) and ( item_pos == size ) )
    then

      move_type = 1;
      
    --end;

    elseif ( ( item_pos_old ==  - 1 ) and ( item_pos >  - 1 ) )
    or     ( ( item_pos_old == size ) and ( item_pos < size ) )
    then

      move_type = 2;

    end;

  --[[
    if     ( ( item_pos_old ==  - 1 ) and ( item_pos == size ) )
    or     ( ( item_pos_old == size ) and ( item_pos ==  - 1 ) )
    then

      move_type = 3;

    end;
  ]]

    --DbgTrace( "INV MOVE: object "..i..": "..item_name..", pos = "..item_pos.." ( old = "..item_pos_old.." ), move_type = "..move_type.."." );

    local event_id = "item_move_"..item_name;
    private.event_anim_end[ event_id ] = {};--{ name = item_name, pos = item_pos };
    private.ItemMoveAnim( hub_name, move_type, move_pos, event_id );

  end;

  ObjSet( "tmr_int_inventory_items_move",
  {
    endtrig = function () private.is_objects_moving = false; end,
    time    = private.ITEM_MOVE_ANIM_TIME,
    playing = true
  } );

end;
--******************************************************************************************
function private.ItemMoveAnim( object_name, move_type, move_pos, event_id )

  local trg = function () private.EventAnimEnd( event_id ); end;

  local tme = private.ITEM_MOVE_ANIM_TIME;

  local obj_params = ObjGet( object_name );

  ObjAnimate( object_name, "pos_x", 0, 0, trg,
  {
    0.0, 3, obj_params.pos_x,
    tme, 3, move_pos
  } );

  if ( move_type == 0 ) then

    --ObjStopAnimate( object_name, 6 );
    --ObjStopAnimate( object_name, 8 );

  elseif ( move_type == 1 ) then

    ObjAnimate( object_name, "scale_xy", 0, 0, "",
    {
      0.0, 2, obj_params.scale_x, obj_params.scale_y,
      tme, 2, 0, 0
    } );

    ObjAnimate( object_name, "alp", 0, 0, "",
    {
      0.0, 2, obj_params.alp,
      tme, 2, 0
    } );

  elseif ( move_type == 2 ) then

    ObjAnimate( object_name, "scale_xy", 0, 0, "",
    {
      0.0, 1, obj_params.scale_x, obj_params.scale_y,
      tme, 1, 1, 1
    } );

    ObjAnimate( object_name, "alp", 0, 0, "",
    {
      0.0, 1, obj_params.alp,
      tme, 1, 1
    } );

  end;

end;
--******************************************************************************************
function private.FlyToCursor( item_name )

  SetCursor( CURSOR_NULL );

  local def_name = interface.GetObjectName( item_name );
  local hub_name = "hub_int_providence_panel_"..def_name;
  local sub_name = "sub_int_providence_panel_"..def_name;
  local obj_name = "itm_int_providence_panel_"..def_name;

  local event_id = "item_drag_"..item_name;
  local trg = function () private.EventAnimEnd( event_id ); end;
  private.event_anim_end[ event_id ] = {};

  private.StdItemDragAnimation( obj_name, sub_name, trg );

end;
--******************************************************************************************
function private.FlyToProvidencePanel( item_name )

  SetCursor( CURSOR_DEFAULT );

  local def_name = interface.GetObjectName( item_name );
  local hub_name = "hub_int_providence_panel_"..def_name;
  local sub_name = "sub_int_providence_panel_"..def_name;
  local obj_name = "itm_int_providence_panel_"..def_name;

  int_inventory.count_fly_animation_playing = int_inventory.count_fly_animation_playing - 1;
  local event_id = "item_drop_"..item_name;
  local trg = function () private.EventAnimEnd( event_id ); end;
  private.event_anim_end[ event_id ] = { name = item_name };

  private.StdItemDropAnimation( obj_name, sub_name, trg, private.ITEM_SCALE );

end;
--******************************************************************************************
function private.ItemCreate( item_name, hub_name )

  local back_name = "spr_int_panel_providence_"..interface.GetObjectName( item_name ).."_back";

  ObjCreate( back_name, "spr" );

  ObjSet( back_name,
  {
  res = "assets/interface/int_inventory/item_back";
  input = false,
  blendmode = 2,
  pos_z = 0
  } );

  ObjAttach( back_name, hub_name );

end;
--******************************************************************************************
function private.StartDrag( item_name )
  int_inventory.count_fly_animation_playing = int_inventory.count_fly_animation_playing + 1;
  private.FlyToCursor( item_name );
  private.m_dragging_object = item_name;  
  ObjSet( item_name, { input = false } );
  SetCursor( CURSOR_NULL );

  if private.m_objects_params[ item_name ] and 
     private.m_objects_params[ item_name ].event_startdrag ~= nil then 
     private.m_objects_params[ item_name ].event_startdrag(); 
  end;

end;
--******************************************************************************************
function private.StopDrag( item_name )

  local def_name = interface.GetObjectName( item_name );
  local obj_name = "itm_int_inventory_"..def_name;

  MsgSend( Command_Transporter_StopAnim, 
  { 
     type = Trans_InventoryToCursor,
     obj_name = obj_name
  } );
  ObjSet( item_name, { input = false } );

  if private.m_objects_params[ item_name ] and 
     private.m_objects_params[ item_name ].event_dragdrop ~= nil then 
     private.m_objects_params[ item_name ].event_dragdrop(); 
  end;
  private.m_dragging_object = nil; 
  SetCursor( CURSOR_DEFAULT );
  if stoptdrag_func ~= nil then stoptdrag_func(); end;
  
  if ObjGet( item_name ) ~= nil then
    private.FlyToProvidencePanel( item_name );
  end;

end;
--******************************************************************************************
function private.ItemMouseEnter( item_name ) 

  private.ZoomIn( item_name ); 
  int_inventory.Show( true ); 

  if private.m_objects_params[ item_name ] and 
     private.m_objects_params[ item_name ].event_menter ~= nil then 
     private.m_objects_params[ item_name ].event_menter(); 
  end;

end
--******************************************************************************************
function private.ItemMouseLeave( item_name ) 

  private.ZoomOut( item_name ); 

  if private.m_objects_params[ item_name ] and 
     private.m_objects_params[ item_name ].event_mleave ~= nil then 
     private.m_objects_params[ item_name ].event_mleave(); 
  end;

end
--******************************************************************************************
function private.InvGetObjPos( idx, start, m )

  if idx < start then
    return -1;
  elseif idx - start < m then
    return idx - start;
  else
    return m;
  end;

end;
--******************************************************************************************
function private.ExecuteAddEndFunc( item_name )

  for i = 1, #private.m_objects do
    if private.m_objects[ i ].name == item_name then
      if private.m_objects[ i ].func then
        private.m_objects[ i ].func();
      end;
      break;
    end;
  end;

end;