-- name=int_providence
--******************************************************************************************
function public.Init ( pos_z )

  ObjAttach( "int_providence", interface.originhub );
  ObjSet( "int_providence", { pos_z = pos_z } );

  public.show = false;

  private.locs = {};
  private.locs[ "fork" ] = {};
  private.locs[ "livingroom" ] = { "get_object_1" };
  private.locs[ "office" ] = { "get_prv_statuette", "get_prv_criminalsbag" };
  private.locs[ "abandoneddistrict" ] = { "get_prv_knife", "get_prv_hat" };
  private.locs[ "mineentrance" ] = { "get_prv_sledgehammer", "get_prv_valve" };
  private.locs[ "nightsvilleentrance" ] = { "get_prv_carrot" };
  private.locs[ "hallway" ] = { "get_prv_curtainrope", "get_prv_flowerpot" };
  private.locs[ "chamber" ] = { "get_prv_photoalbum", "get_prv_figurine" };
  private.locs[ "wincards" ] = { "get_prv_handcuffs", "get_prv_fuellamp" };
  private.locs[ "secretroom" ] = { "get_prv_gun", "get_prv_gnomehead" };
  private.thinking_objs = {};
  private.thinking_objs.delete = {};
  private.thinking_objs.input = {};
  private.objects = {};

end;
--******************************************************************************************
function public.Destroy()


end;
--******************************************************************************************
function public.GetEvents( room )

  return private.locs[ room ];

end;
--******************************************************************************************
function private.EventAnimEnd( event_id )

  common_impl.Lock( false );

  if ( event_id == "show" ) then

    common_impl.StartProvidence();

    if not public.GetEvents( common.GetObjectName( common.GetCurrentRoom() ) ) then

      common_impl.ShowBbt( 'bbt_providence_notuse' );

    end;

  elseif ( event_id == "hide" ) then

    public.show = false;

    local room = GetCurrentRoom();

    ObjDetach( room.."_providence" );

    common_impl.StopProvidence();

    ObjSet( "obj_int_providence", { active = false, visible = false, input = false } );

    for i = 1, #private.objects do

      ObjSet( private.objects[ i ], { input = true } )
      for j = 1, 3 do
        ObjDelete( "obj_int_providence_thinking_"..private.objects[ i ].."_"..j );
      end;

    end;

    private.objects = {};


  end;

end;
--******************************************************************************************
function private.ShowAnim( event_id )

  public.show = true;

  local obj = "obj_int_providence";
  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = 0.3;

  local room = GetCurrentRoom();

  ObjAttach( room.."_providence", "obj_int_providence" );

  common_impl.PlayAudio( "sfx", "common/audio/aud_providence_activate", 0, "", 0 );

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 2, 0,
    tme, 2, 1
  } );

end;
--******************************************************************************************
function private.HideAnim( event_id )

  local obj = "obj_int_providence";
  local trg = function () private.EventAnimEnd( event_id ); end;
  local tme = 0.3;

  common_impl.PlayAudio( "sfx", "common/audio/aud_providence_deactivate", 0, "", 0 );

  ObjAnimate( obj, "alp", 0, 0, trg,
  {
    0.0, 2, 1,
    tme, 2, 0
  } );

end;
--******************************************************************************************
function public.WideScreenUpdate()

  if ( not GetWideScreen() ) then

    --
  
  else

    --

  end;

end;
--******************************************************************************************
function public.Show()

  local room = GetCurrentRoom();

  if ( not string.find( room, "rm_" )  or ( interface.GetCurrentInv() ~= "" ) ) then --временно, пока нет блокировок кнопок в не локациях

    return;

  end;

  common_impl.Lock( true );

  int_button_hint_impl.ChangeToProvidence( true );

  common.CloseSubRoom();

  ObjSet( "obj_int_providence", { active = true, visible = true, input = true } );

  local event_id = "show";

  private.ShowAnim( event_id );

end;
--******************************************************************************************
function public.Hide()

  private.StopThinking();

  common_impl.Lock( true );

  private.GetAllSilhouette( common.GetObjectName( common.GetCurrentRoom() ) );
  int_button_hint_impl.ChangeToProvidence( false );

  local event_id = "hide";
  private.HideAnim( event_id );

end;
--******************************************************************************************
function private.GetAllSilhouette( room )

  local events = public.GetEvents( room )

  if not events then

    return;

  end;

  for i = 1, #events do

    local event = events[ i ];
    local new_event = events[ i ].."_silhouette";

    if not ( cmn.IsEventDone( event ) or cmn.IsEventDone( new_event ) ) then
      common_impl.PlayAudio( "sfx", "common/audio/aud_providence_get_empty", 0, "", 0 );
      cmn.SetEventDone( new_event );
      cmn.CallEventHandler( new_event );

    end;

  end;

end;
--******************************************************************************************
function public.Thinking( name, object, func )

  local delay_tm = 3;
  local pos = GetGameCursorPos(); 
 
  local func_execute = function()
     
     ObjSet( object, { input = true } )

    for i = 1, 3 do

      ObjDelete( "obj_int_providence_thinking_"..object.."_"..i );

    end;

    if func then

      func();

    end;
  
    for i = 1, #private.objects do
      if ( private.objects[ i ] == object ) then
        table.remove( private.objects, i );
        break;
      end;
    end;

  end;

  common_impl.PlayAudio( "sfx", "common/audio/aud_deduction_words", 0, "", 0 ); 

  if not common_impl.ExistInArr( private.thinking_objs.input, object ) then
  
    table.insert( private.thinking_objs.input, object );

  end;


  table.insert( private.objects, object );

  for i = 1, 3 do

    local ang = math.random( 78, 235 ) / 100;

    ObjClone( "obj_int_providence_thinking", "_"..object.."_"..i, 1, "obj_int_providence" );
    ObjSet( "obj_int_providence_thinking_"..object.."_"..i, { pos_x = pos[ 1 ], pos_y = pos[ 2 ], ang = ang } );
    ObjSet( "txt_int_providence_thinking_"..object.."_"..i, { ang = -ang, text = "str_"..common.GetObjectName( GetCurrentRoom() ).."_"..name.."_"..i } );
    ObjSet( object, { input = false } )
    
    local delay = delay_tm * ( i - 1 );
    local func_end = "";
    if ( i == 3 ) then
      func_end = func_execute;
    end;

    ObjAnimate( "txt_int_providence_thinking_"..object.."_"..i, "pos_x", 0, 0, func_end, { 0, 0, 0,   delay, 0, 0,    ( delay + delay_tm ), 0, 100 } );
    ObjAnimate( "obj_int_providence_thinking_"..object.."_"..i, "scale_xy", 0, 0, "", { 0, 0, 1, 1,   delay, 0, 1, 1,    ( delay + delay_tm/2 ), 3, 1.2, 1.2,   ( delay + delay_tm ), 3, 1, 1, } );
    ObjAnimate( "obj_int_providence_thinking_"..object.."_"..i, "alp", 0, 0, "", { 0, 0, 0,   delay, 0, 0,    ( delay + delay_tm/2 ), 0, 1,   ( delay + delay_tm ), 0, 0 } );

    if not common_impl.ExistInArr( private.thinking_objs.delete, "txt_int_providence_thinking_"..object.."_"..i) then

      table.insert( private.thinking_objs.delete, "txt_int_providence_thinking_"..object.."_"..i );

    end;

    if not common_impl.ExistInArr( private.thinking_objs.delete, "obj_int_providence_thinking_"..object.."_"..i ) then

      table.insert( private.thinking_objs.delete, "obj_int_providence_thinking_"..object.."_"..i );

    end;

  end;


end;

function private.StopThinking()

  local table_multiset = {};

  for i = 1, #private.thinking_objs.delete  do

    ObjDelete( private.thinking_objs.delete[i] );

  end;

  private.thinking_objs.delete = {};

  for i = 1, #private.thinking_objs.input do

    table.insert( table_multiset, { private.thinking_objs.input[i], { input = true } });

  end;
  
  private.thinking_objs.input = {};

  if #table_multiset > 0 then
   
      ObjMultiSet( table_multiset );

  end;


end;