-- name=ho_littlehouse
--*********************************************************************************************************************
function public.Init()
  --*********************************************************************************************************************------
  room.Define( "littlehouse" );
  game.relations[ "ho_littlehouse" ].exitroom = "rm_glade";  

  private.soldiers_to_objects = 
  {
    {"bumerang", "owl", "teeth"},
    {"dreidel", "pencils", "train"},
    {"hat", "slingshot", "sword"},
    {"knife", "snail", "yoyo"}
  }

  local table_multiset = {

    { "spr_littlehouse_bumerang",   { event_mdown = function() private.TaskComplete( 'bumerang', 1 ) end } },
    { "spr_littlehouse_dreidel",    { event_mdown = function() private.TaskComplete( 'dreidel', 2 ) end } },
    { "spr_littlehouse_hat",        { event_mdown = function() private.TaskComplete( 'hat', 3 ) end } },
    { "spr_littlehouse_knife",      { event_mdown = function() private.TaskComplete( 'knife', 4 ) end } },
    { "spr_littlehouse_owl",        { event_mdown = function() private.TaskComplete( 'owl', 1 ) end } },
    { "spr_littlehouse_pencils",    { event_mdown = function() private.TaskComplete( 'pencils', 2 ) end } },
    { "spr_littlehouse_slingshot",  { event_mdown = function() private.TaskComplete( 'slingshot', 3 ) end } },
    { "spr_littlehouse_snail",      { event_mdown = function() private.TaskComplete( 'snail', 4 ) end } },
    { "spr_littlehouse_teeth",      { event_mdown = function() private.TaskComplete( 'teeth', 1 ) end } },
    { "spr_littlehouse_train",      { event_mdown = function() private.TaskComplete( 'train', 2 ) end } },
    { "spr_littlehouse_sword",      { event_mdown = function() private.TaskComplete( 'sword', 3 ) end } },    
    { "spr_littlehouse_yoyo",       { event_mdown = function() private.TaskComplete( 'yoyo', 4 ) end } },
    { "obj_littlehouse_taskpanel_use_lockpart", { event_mdown = function() common_impl.ShowBbt( 'bbt_littlehouse_use_lockpart' ); end } }

  };
    
  for i=1, 4 do

    table.insert( table_multiset, { "spr_littlehouse_soldier_"..i, { event_startdrag = function() private.GetSoldier( i ) end, event_dragdrop = function() private.UseSoldier( i ) end } });
    table.insert( table_multiset, { "spr_littlehouse_taskpanel_part_"..i, { event_startdrag = function() private.GetPart( i ) end, event_dragdrop = function() private.UsePart( i ) end } });

  end;

  ObjMultiSet( table_multiset );

--******ANIMS**********************************************************************************************************
  ObjAnimate("spr_littlehouse_plane1", "ang", 1, 0, "", {0.0,0,-0.1,  1.9,3,0.15, 3.8,3,-0.1});
  ObjAnimate("spr_littlehouse_plane2", "ang", 1, 0, "", {0.0,0,0.1,  1.6,3,-0.1, 3.2,3,0.1});
  ObjAnimate("spr_littlehouse_light_1", "alp", 1, 0, "", {0.0,0,1,  0.5,0,0.8,  0.9,0,0.9,  1.2,0,0.7,  1.8,0,1});
  ObjAnimate("spr_littlehouse_light_2", "alp", 1, 0, "", {0.0,0,1,  1.2,0,0.8,  1.5,0,0.7,  2.5,0,1});
--*********************************************************************************************************************
  --------------------------------------------------------------------------------------------
  -- function UpdateGameMode () end;
  --------------------------------------------------------------------------------------------
  local update_game_mode_littlehouse = function ()

    local visible_sparkle_ho = true;  
    if ( ng_global.gamemode == 3 and not ng_global.gamemode_custom[ "sparkle_ho" ] ) then    
      visible_sparkle_ho = false;
    end;

    local mas_parent_gfx = 

      {
        "ho_littlehouse",
      };

    common_impl.TurnChildGfx( mas_parent_gfx, "ho_littlehouse", visible_sparkle_ho );

  end;
  cmn.AddSubscriber( "update_game_mode_littlehouse", update_game_mode_littlehouse, "ho_littlehouse" );
  ---------------------------------------------------------------------------------------------------
  common_impl.hint[ "win_littlehouse" ] = {
      type = "win",
      room = "rm_glade",
      zz = "ho_littlehouse",
      zz_gate = "gho_glade_littlehouse"
  };
  
  local win_littlehouse_func = function ()

    if ( cmn.IsEventDone( "win_littlehouse" ) ) then

      interface.InventoryItemAdd( "inv_camera", "spr_littlehouse_win" );
      ObjSet( "inv_camera", { drag = false });

    else

      local ho_found = ng_global.progress[ "std" ][ "win_littlehouse" ].found;
        
      for i = 1, #ho_found do

        ObjDelete( ho_found[ i ][ "object" ] ) 
        ObjDelete("txt_littlehouse_taskpanel_task_"..ho_found[ i ][ "task" ]);

      end;

      local task;

      for i = 1, 4 do

        if ng_global.progress[ "std" ][ "win_littlehouse" ].parts_done[ i ] == 3 then

          ObjSet("spr_littlehouse_taskpanel_part_"..i, {alp = 1, input = true});
          ObjSet("obj_littlehouse_taskpanel_hint_lockpart"..i, {input = true});
          ObjDelete("obj_littlehouse_taskpanel_soldier_"..i.."_txt");

        end;

        if ng_global.progress[ "std" ][ "win_littlehouse" ].parts_used[ i ] == 1 then

          ObjSet("spr_littlehouse_lockpart_"..i, {alp = 1});
          ObjDelete("obj_littlehouse_taskpanel_hint_lockpart"..i);

        end;

        if ( ng_global.progress[ "std" ][ "win_littlehouse" ].soldiers_done[ i ] == 1 ) then

          --

        end;

        if ( ng_global.progress[ "std" ][ "win_littlehouse" ].opened_panel[ i ] == 1 ) then

          ObjDelete( "obj_littlehouse_taskpanel_hint_soldier"..i );
          ObjSet( "obj_littlehouse_taskpanel_soldier_"..i.."_txt", { alp = 1 } );
          ObjDelete( "spr_littlehouse_taskpanel_soldier_"..i );
                  
          for j = 1, 3 do

            task = private.soldiers_to_objects[ i ][ j ]
            if ( ObjGet( "spr_littlehouse_"..task ) ) then

              ObjSet( "spr_littlehouse_"..task, { input = true } );

            end;

          end;

        end;

      end;
            
    end;

  end;
  cmn.AddSubscriber( "win_littlehouse", win_littlehouse_func );

end;
--*********************************************************************************************************************
function public.Load()

end;
--*********************************************************************************************************************
function public.PreOpen()

  if ( not cmn.IsEventDone( "opn_littlehouse" ) ) then
  
    cmn.InitHo( "littlehouse" );
    cmn.SetEventDone( "opn_littlehouse" );

    ng_global.progress[ "std" ][ "win_littlehouse" ].opened_panel   = { 0, 0, 0, 0 };
    ng_global.progress[ "std" ][ "win_littlehouse" ].soldiers_done  = { 0, 0, 0, 0 };
    ng_global.progress[ "std" ][ "win_littlehouse" ].parts_done     = { 0, 0, 0, 0 };
    ng_global.progress[ "std" ][ "win_littlehouse" ].parts_used     = { 0, 0, 0, 0 };
  
  end;

  local ho_items = {};

  interface.TaskPanelTaskAdd( ho_items, "soldier1",           1, 0, { { "spr_littlehouse_soldier_1", "obj_littlehouse_taskpanel_hint_soldier1" } } );
  interface.TaskPanelTaskAdd( ho_items, "soldier2",           1, 0, { { "spr_littlehouse_soldier_2", "obj_littlehouse_taskpanel_hint_soldier2" } } );
  interface.TaskPanelTaskAdd( ho_items, "soldier3",           1, 0, { { "spr_littlehouse_soldier_3", "obj_littlehouse_taskpanel_hint_soldier3" } } );
  interface.TaskPanelTaskAdd( ho_items, "soldier4",           1, 0, { { "spr_littlehouse_soldier_4", "obj_littlehouse_taskpanel_hint_soldier4" } } );
 
  interface.TaskPanelTaskAdd( ho_items, "bumerang",           1, 0, { { "spr_littlehouse_bumerang" } } );
  interface.TaskPanelTaskAdd( ho_items, "dreidel",            1, 0, { { "spr_littlehouse_dreidel" } } );
  interface.TaskPanelTaskAdd( ho_items, "hat",                1, 0, { { "spr_littlehouse_hat" } } );
  interface.TaskPanelTaskAdd( ho_items, "knife",              1, 0, { { "spr_littlehouse_knife" } } );  
  interface.TaskPanelTaskAdd( ho_items, "owl",                1, 0, { { "spr_littlehouse_owl" } } );
  interface.TaskPanelTaskAdd( ho_items, "pencils",            1, 0, { { "spr_littlehouse_pencils" } } );
  interface.TaskPanelTaskAdd( ho_items, "slingshot",          1, 0, { { "spr_littlehouse_slingshot" } } );
  interface.TaskPanelTaskAdd( ho_items, "snail",              1, 0, { { "spr_littlehouse_snail" } } );
  interface.TaskPanelTaskAdd( ho_items, "teeth",              1, 0, { { "spr_littlehouse_teeth" } } );
  interface.TaskPanelTaskAdd( ho_items, "train",              1, 0, { { "spr_littlehouse_train" } } );
  interface.TaskPanelTaskAdd( ho_items, "sword",              1, 0, { { "spr_littlehouse_sword" } } );
  interface.TaskPanelTaskAdd( ho_items, "yoyo",               1, 0, { { "spr_littlehouse_yoyo" } } );

  interface.TaskPanelTaskAdd( ho_items, "part1",              1, 0, { { "spr_littlehouse_taskpanel_part_1", "obj_littlehouse_taskpanel_hint_lockpart1" } } );
  interface.TaskPanelTaskAdd( ho_items, "part2",              1, 0, { { "spr_littlehouse_taskpanel_part_2", "obj_littlehouse_taskpanel_hint_lockpart2" } } );
  interface.TaskPanelTaskAdd( ho_items, "part3",              1, 0, { { "spr_littlehouse_taskpanel_part_3", "obj_littlehouse_taskpanel_hint_lockpart3" } } );
  interface.TaskPanelTaskAdd( ho_items, "part4",              1, 0, { { "spr_littlehouse_taskpanel_part_4", "obj_littlehouse_taskpanel_hint_lockpart4" } } );

  interface.TaskPanelShow( ho_items );
  int_taskpanel_impl.HideTextPanel( "littlehouse" );

  ObjSet( "obj_littlehouse_taskpanel", { visible = true, input = true, pos_z = 1, pos_x = -190, pos_y = 47 } ); 

  common_impl.PlayAudio( "mus", "common/audio/aud_ho_mus", 1, "", 0.5 );

  interface.InvestigationButtonCheck();

end;
--*********************************************************************************************************************
function public.Open ()
  
  common_impl.VisitMapRoom( "glade" );
  common_impl.PassingHoStart();
  common_impl.TutorialShow( "tutorial_page_24" );

end;
--*********************************************************************************************************************
function public.PreClose()

  interface.TaskPanelHide();

end;
--*********************************************************************************************************************
function public.Close ()

  common_impl.PassingHoStop();
  int_taskpanel_impl.ShowTextPanel( "littlehouse" );
  common_impl.StopAudio( "mus", "common/audio/aud_ho_mus", 1, "", 0.5 );

end;
--*********************************************************************************************************************
function public.HoComplete ()

  common_impl.PassingHoWin();
    
end;
--*********************************************************************************************************************
--*********************************************************************************************************************
function private.get_camera ()

  cmn.SetEventDone( "win_littlehouse" );
  cmn.CallEventHandler( "win_littlehouse" );
  cmn.GotoRoom( "rm_glade" );

end;
--*********************************************************************************************************************
function private.TaskComplete ( task_name, part_num )

  local object = "spr_littlehouse_"..task_name;

  
  ObjSet( object, { pos_z = 1 } );

  cmn.HoTaskFind( object, task_name );
  ObjDelete("txt_littlehouse_taskpanel_task_"..task_name);

  ng_global.progress[ "std" ][ "win_littlehouse" ].parts_done[part_num] = ng_global.progress[ "std" ][ "win_littlehouse" ].parts_done[part_num] + 1;

  if ( ng_global.progress[ "std" ][ "win_littlehouse" ].parts_done[part_num] == 3 ) then

    ObjAnimate("spr_littlehouse_taskpanel_part_"..part_num, "alp", 0, 0, "", {0.0,0,0,  0.3,0,1});    
    ObjAnimate("obj_littlehouse_taskpanel_soldier_"..part_num.."_txt", "alp", 0, 0, "", {0.0,0,1,  0.3,0,0});
    ObjSet("spr_littlehouse_taskpanel_part_"..part_num, {input = true});
    ObjSet( "obj_littlehouse_taskpanel_hint_lockpart"..part_num, { input = true } );

  end;
  
end;
--*********************************************************************************************************************
function private.GetSoldier ( num )

  common_impl.PlayAudio("sfx","common/audio/aud_get_inv_sfx");
  ObjAttach("spr_littlehouse_soldier_"..num, "obj_littlehouse_taskpanel");
  ObjSet( "spr_littlehouse_soldier_"..num, { pos_z = 30 } );  
  ObjSet( "spr_littlehouse_soldier_"..num.."_shadow", { visible = false } );  

end;
--*********************************************************************************************************************
function private.UseSoldier ( num )

  common_impl.SetCursor( CURSOR_DEFAULT );
  local use_panel = nil;

  for i = 1, 4 do

    if ( common.ApplyObj( "spr_littlehouse_soldier_"..num, "obj_littlehouse_taskpanel_use_soldier_"..i ) ) then

      use_panel = i;
      break;

    end;

  end;

  if ( use_panel ) then

    common_impl.TutorialSetShown("tutorial_page_24");
    common_impl.Lock( true );

    local func_end = function ()

      common_impl.Lock( false );

      cmn.HoTaskFind( "spr_littlehouse_soldier_"..num, "soldier"..num, true );

      ng_global.progress[ "std" ][ "win_littlehouse" ].soldiers_done[ num ] = 1;
      ng_global.progress[ "std" ][ "win_littlehouse" ].opened_panel[ use_panel ] = 1;

      private.OpenObjects( num, use_panel );

    end;

    common_impl.PlayAudio( "sfx", "common/audio/aud_ho_littlehouse_soldier", 0, "", 0 );

    ObjSet( "tmr_littlehouse_sound", { time = 0.3, playing = true, endtrig = "*common_impl.PlayAudio( 'sfx', 'common/audio/aud_ho_panel_off', 0, '', 0 );" } );

    ObjAnimate( "spr_littlehouse_taskpanel_soldier_"..use_panel.."_done", "alp", 0, 0, "", 
    {
      0.0,0,0,
      0.3,0,1
    } );

    ObjAnimate( "spr_littlehouse_taskpanel_soldier_"..use_panel, "pos_y", 0, 0, func_end, 
    {
      0.0,0,1,
      0.3,0,1,
      1.6,0,90
    } );

    ObjAnimate( "obj_littlehouse_taskpanel_soldier_"..use_panel.."_txt", "alp", 0, 0, "", 
    {
      0.0,0,0,
      0.3,0,1
    } );

  else

    ObjAttach( "spr_littlehouse_soldier_"..num, "ho_littlehouse");
    ObjSet( "spr_littlehouse_soldier_"..num, { pos_z = 2 } );
    ObjSet( "spr_littlehouse_soldier_"..num.."_shadow", { visible = true } ); 

  end;

end;
--*********************************************************************************************************************
function private.OpenObjects ( sold_num, panel_num )

  ObjDelete( "spr_littlehouse_taskpanel_soldier_"..panel_num );
  ObjDelete( "obj_littlehouse_taskpanel_use_soldier_"..panel_num );
  ObjDelete( "obj_littlehouse_taskpanel_hint_soldier"..panel_num);
  ObjSet( "obj_littlehouse_taskpanel_soldier_"..panel_num.."_txt", { alp = 1 } );

  for i = 1, 3 do

    ObjSet( "spr_littlehouse_"..private.soldiers_to_objects[ panel_num ][ i ], { input = true } );

  end;

end;
--*********************************************************************************************************************
function private.GetPart ( num )

  common_impl.PlayAudio("sfx","common/audio/aud_get_inv_sfx");
  ObjSet( "spr_littlehouse_taskpanel_part_"..num, { pos_z = 30 } );  

end;
--*********************************************************************************************************************
function private.UsePart ( num )

  common_impl.SetCursor( CURSOR_DEFAULT );

  if ( common.ApplyObj( "spr_littlehouse_taskpanel_part_"..num, "obj_littlehouse_taskpanel_use_lockpart" ) ) then

    common_impl.Lock(true);

    local func_end = function ()

      common_impl.Lock(false);

      ng_global.progress[ "std" ][ "win_littlehouse" ].parts_used[num] = 1;

      local fin = true;

      for i=1, 4 do

        if (ng_global.progress[ "std" ][ "win_littlehouse" ].parts_used[ i ] == 0) then

          fin = false;

        end;

      end;

      if (fin) then

        private.WinAnim(num);

      else

        cmn.HoTaskFind( "spr_littlehouse_taskpanel_part_"..num, "part"..num, true );
        ObjDelete("spr_littlehouse_taskpanel_part_"..num);
        ObjDelete("obj_littlehouse_taskpanel_hint_lockpart"..num);

      end;

    end;

    common_impl.PlayAudio( "sfx", "common/audio/aud_ho_littlehouse_insert", 0, "", 0 );

    ObjSet("spr_littlehouse_taskpanel_part_"..num, {alp = 0});
    ObjAnimate("spr_littlehouse_lockpart_"..num, "alp", 0, 0, func_end, 
    {
      0.0,0,0,
      0.3,0,1
    }); 
    
  else

    ObjSet( "spr_littlehouse_taskpanel_part_"..num, { pos_z = 3 } );
        
  end;
  

end;
--*********************************************************************************************************************
function private.WinAnim(num)

  local func_after = function()

    common_impl.Lock(false);
    cmn.HoTaskFind( "spr_littlehouse_taskpanel_part_"..num, "part"..num, true );    

  end;

  common_impl.Lock(true);

  common_impl.PlayAudio( "sfx", "common/audio/aud_ho_littlehouse_chest_open", 0, "", 0 );
  
  ObjDelete("spr_littlehouse_taskpanel_part_"..num);
  ObjAnimate("spr_littlehouse_chest_closed", "alp", 0, 0, "", {0.0,0,1, 0.3,0,0});
  ObjAnimate("obj_littlehouse_chest_open", "alp", 0, 0, func_after, {0.0,0,0, 0.3,0,1,  0.6,0,1});

end;
--*********************************************************************************************************************
